<?php
namespace Teknisa\PLA\Modules\Cardapio\Controllers;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\GER\Utilities\Environment;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO;
use \Setforms2\Shared\ORM\DateTime;
use \Doctrine\ORM\EntityManager;

class Pla06600SubstPratoCardaController extends \Teknisa\GER\Controllers\TeknisaCrud {

    private   $entityManager;
    protected $crudService;
    protected $dataSourceManager;
    protected $environment;
    protected $connection;
    protected $parameterBag;
    protected $substPratoCardaService;
    protected $dataSourceParamBag;
    protected $nrlista_fili;
    protected $nrlista_filiserv;
    protected $params;
    protected $idutcardpadr;
    protected $grupoperador;
    protected $libCardapio;

    public function __construct($crudService, $parameterBag, $entityManager, Manager $dataSourceManager, Environment $environment, $connection, $dataSourceParamBag, $substPratoCardaService, $util) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->entityManager = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
        $this->connection = $connection;
        $this->dataSourceParamBag = $dataSourceParamBag;
        $this->substPratoCardaService = $substPratoCardaService;
        $this->util = $util;
	}

    public function validacaoGeral(DTO\Request $request, DTO\Response $response) {
        try {
            set_time_limit (300);
            $row = $request->getDataSet()->getRows();
            $this->createParams($row);
            $this->verificaParamgeral();
            $this->insereListas($this->params["FILIAIS"], $this->params["FILIAIS_SERVICOS"]);
            $result = $this->checkActiveRecipe($row);
            if(!empty($result)){
                $response->addDataSet(new DataSet ('receitaInativa', array($result)));
                throw new \Exception("Existem receitas inativas. Verifique o log.");
            }
            $this->verificaExistenciaCardapios();
            $this->verificaDataProc();
            $this->excluiLista($this->nrlista_fili);
            $this->excluiLista($this->nrlista_filiserv);

        } catch (\Exception $e) {
            $this->excluiLista($this->nrlista_fili);
            $this->excluiLista($this->nrlista_filiserv);
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function validacaoLibCarda(DTO\Request $request, DTO\Response $response) {
        /*
            Definido até o momento que todos se comportarão como matriz,
            visto que as unidades exibidas são apenas as que o operador tem acesso

            try {
                $this->createParams($request->getDataSet()->getRows());
                $this->verificaGrupOperador();
                $this->verificaLibCarda();
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), 1));

            }
        */
    }

    public function confirmaSubstPratoCarda(DTO\Request $request, DTO\Response $response) {
        try {
            $this->createParams($request->getDataSet()->getRows());
            $this->insereListas($this->params["FILIAIS"], $this->params["FILIAIS_SERVICOS"]);
            $nrlogintcard = $this->procedureTrocaPratoCard();
            $this->excluiLista($this->nrlista_fili);
            $this->excluiLista($this->nrlista_filiserv);

            $response->addDataSet(new DataSet ('nrlogintcard', array($nrlogintcard)));

        } catch (\Exception $e) {
            $this->excluiLista($this->nrlista_fili);
            $this->excluiLista($this->nrlista_filiserv);
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function createParams($params) {
        $this->params = array(
            "DADOS" => $params[0],
            "FILIAIS" => $params[1],
            "FILIAIS_SERVICOS" => $params[2]
        );
    }

    public function insereListas($arrayFiliais, $arrayFiliaisServicos) {
        set_time_limit (300);
        $this->nrlista_fili = $this->util->insertLista($arrayFiliais);
        $this->nrlista_filiserv = $this->util->insertLista($arrayFiliaisServicos);
    }

    public function excluiLista($nrlista) {
        $this->substPratoCardaService->excluiLista($nrlista);
    }

    public function verificaParamgeral() {

        $cdpargeral = '0001';
        $nrorg = $this->environment->getNrOrg();
        $this->parameterBagColumns = array('CDPARGERAL', 'NRORG');

        $filterCriteria = $this->substPratoCardaService->verificaParamgeral($cdpargeral, $nrorg);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria,$this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

        $this->idutcardpadr = !empty($dataSet->getRows()) ? $dataSet->getRows()[0]['IDUTCARDPADR'] : NULL;
    }

    public function verificaExistenciaCardapios() {
        $datetime = new \DateTime();

        $listaserv = $this->nrlista_filiserv;
        $listafili = $this->nrlista_fili;
        $dtinicial = $this->params["DADOS"]["INTERVAL_start"];
        $dtfinal = $this->params["DADOS"]["INTERVAL_end"];

        if($datetime->createFromFormat('d/m/Y', $dtinicial) > $datetime->createFromFormat('d/m/Y', $dtfinal)){
            throw new \Exception("A Data Inicial deve ser igual ou inferior a Data Final.");
        }

        $cdprato = $this->params["DADOS"]["CDPRATOATUAL"];
        $cdfilaux = $this->params["DADOS"]["CDFILAUXPRATATUAL"];
        $nrorg = $this->environment->getNrOrg();
        $this->parameterBagColumns = array('LISTASERV', 'LISTAFILI', 'DTINICIAL', 'DTFINAL', 'CDPRATO', 'CDFILAUX', 'NRORG');

        $filterCriteria = $this->substPratoCardaService->verificaExistenciaCardapios($listaserv, $listafili, $dtinicial, $dtfinal, $cdprato, $cdfilaux, $nrorg);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria,$this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
       
        
        $arrayFiliaisServicos = array();
        $arrayFiliais = array();
        foreach($dataSet->getRows() as $row){
            array_push($arrayFiliaisServicos, $row['CDSERVFILI']);
            array_push($arrayFiliais,  substr($row['CDSERVFILI'], 0, 4));
        }
        $arrayFiliais = array_unique($arrayFiliais);
        $this->params["FILIAIS_SERVICOS"] = $arrayFiliaisServicos;
        $this->params["FILIAIS"] = $arrayFiliais;
        if (empty($dataSet->getRows())) {
            throw new \Exception("Prato atual não existe nos cardápios informados.");
        }
    }

    public function verificaDataProc() {

        $listafili = $this->nrlista_fili;
        $cdpargeral = '0001';
        $dtinicial = $this->params["DADOS"]["INTERVAL_start"];
        $nrorg = $this->environment->getNrOrg();
        $this->parameterBagColumns = array('NRLISTA', 'CDPARGERAL', 'DTINICIAL', 'NRORG');

        $filterCriteria = $this->substPratoCardaService->verificaDataProc($listafili, $cdpargeral, $dtinicial, $nrorg);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        
        if (!empty($dataSet->getRows())) {
            throw new \Exception("O período informado está anterior a data de processamento da unidade.");
        }
    }

    public function verificaGrupOperador() {

        $cdoperador = $this->environment->getCdOperador();
        $nrorg = $this->environment->getNrOrg();
        $this->parameterBagColumns = array('CDOPERADOR', 'NRORG');

        $filterCriteria = $this->substPratoCardaService->verificaGrupOperador($cdoperador, $nrorg);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria,$this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

        $this->grupoperador = !empty($dataSet->getRows()) ? $dataSet->getRows()[0] : NULL;
    }

    public function verificaLibCarda() {

        $cdfilial = $this->params["FILIAIS"][0];
        $cdservfili = $this->params["FILIAIS_SERVICOS"][0];
        $dtmescard = $this->params["DADOS"]["INTERVAL_start"];
        $nrorg = $this->environment->getNrOrg();
        $this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'DTMESCARD', 'NRORG');

        $filterCriteria = $this->substPratoCardaService->verificaLibCarda($cdfilial, $cdservfili, $dtmescard, $nrorg);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria,$this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

        $this->libCardapio = !empty($dataSet->getRows()) ? $dataSet->getRows()[0] : NULL;
    }

    public function procedureTrocaPratoCard() {
        $listafili = $this->nrlista_fili;
        $listaserv = $this->nrlista_filiserv;
        $dtinicial = $this->params["DADOS"]["INTERVAL_start"];
        $dtfinal = $this->params["DADOS"]["INTERVAL_end"];
        $cdpratorig = $this->params["DADOS"]["CDPRATOATUAL"];
        $cdfapratorig = $this->params["DADOS"]["CDFILAUXPRATATUAL"];
        $cdpratdest = $this->params["DADOS"]["CDPRATOSUBST"];
        $cdfapratdest = $this->params["DADOS"]["CDFILAUXPRATSUBST"];
        $modoproc = "M"; /* Definido até o momento que todos se comportarão como matriz visto que as unidades exibidas são apenas as que o operador tem acesso*/
        $cdnvproduto = $this->params["DADOS"]["CDNVPRODUTO"];
        $cdoperador = $this->environment->getCdOperador();
        $nrlogintcard = '';

        $nrlogintcard = $this->substPratoCardaService->TROCA_PRATO_CARD($listafili, $listaserv, $dtinicial, $dtfinal, $cdpratorig, $cdfapratorig, $cdpratdest, $cdfapratdest, $modoproc, $cdnvproduto, $cdoperador, $nrlogintcard);
        return $nrlogintcard['P_NRLOGINTCARD'];
    }
    
    public function checkActiveRecipe($row) {
        return array_merge($this->substPratoCardaService->checkActiveRecipe($row, $this->nrlista_filiserv, 'S'), $this->substPratoCardaService->checkActiveRecipe($row, $this->nrlista_filiserv, 'P'), $this->substPratoCardaService->checkActiveRecipe($row, $this->nrlista_filiserv, 'F'));
    }
}