<?php
namespace Teknisa\PLA\Modules\Prato\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class LimpepraProduto extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = "limpepra_produto";
    protected $dataSourceNameToSave = "limpepra";
    protected $parameterBagColumns = ["CDFILIAL, CDSERVFILI, CDPRATO, NRORG"];
    protected $parameterBag;

    public function __construct($crudService, $parameterBag, $dataSourceManager) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
    }

    public function save(DTO\Request $request, DTO\Response $response) {
    	$dataSet = $request->getDataSet();
    	$rows = $dataSet->getRows();
    	if ($rows[0]['__is_new']) {
    		if ($rows[0]['tipoDeAdicao'] == 'prato') {
    			$registrosExistentes = $this->verificaPratoExiste($rows[0]);

		    	if(empty($registrosExistentes)){
		    		parent::save($request, $response);
		    	} else {
		    		$response->setError(new Error("Este prato já está cadastrado na tabela!", 1));
		    	}
    		} else {
    			$registrosExistentes = $this->verificaProdutoExiste($rows[0]);

		    	if(empty($registrosExistentes)){
		    		parent::save($request, $response);
		    	} else {
		    		$response->setError(new Error("O produto já está cadastrado no prato selecionado!", 1));
		    	}
    		}
   		} else {
   			parent::save($request, $response);
   		}
    }

    public function verificaPratoExiste($row) {
    	return $this->findPratoProduto($row);
    }

    public function verificaProdutoExiste($row) {
    	$condition = array(
			"value" => $row["CDPRODUTO"],
			"operator" => "=",
			"columnName" => "CDPRODUTO"
		);
    	return $this->findPratoProduto($row, $condition);
    }

    public function findPratoProduto($row, $condition = null) {
    	$conditions = array(
    		array(
    			"value"      => $row["CDSERVFILI"],
    			"operator"   => "=",
    			"columnName" => "CDSERVFILI"
    		),
    		array(
    			"value"      => $row["CDFILIAL"],
    			"operator"   => "=",
    			"columnName" => "CDFILIAL"
    		),
    		array(
    			"value"      => $row["CDPRATO"],
    			"operator"   => "=",
    			"columnName" => "CDPRATO"
    		),
    		array(
    			"value"      => $row["NRORG"],
    			"operator"   => "=",
    			"columnName" => "NRORG"
    		)
    	);

    	if ($condition) {
    		$conditions[] = $condition;
    	}

    	$filterCriteria = new FilterCriteria($this->dataSourceName, $conditions);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
    	$dataSetRegExist = $this->dataSourceManager->findBy($newFilterCriteria);

    	return $dataSetRegExist->getRows();
    }
}